import csv
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import pickle

fin_nm = input('Input file name (*.csv) = ')
with open(fin_nm, 'r') as f:
    csv_data = [d for d in csv.reader(f)]

savefig_nm = 'Figure.png'
fout_nm = 'Results.txt' 
fout = open(fout_nm, 'w')
fout.write('Input data file = {}'.format(fin_nm))

N = len(csv_data) - 1
print('N =', N)
X = []
Y = []
Z = []
for i in range(N):
    X.append(float(csv_data[i+1][1]))
    Y.append(float(csv_data[i+1][2]))
    Z.append(float(csv_data[i+1][3]))

fout.write('\n\nData =\n')
for x, y, z in zip(X, Y, Z):
    fout.write(f'{x}  {y}  {z}\n')

print('Compiling the stan script for the correlation coefficients.')
sm0 = pystan.StanModel(file = 'LKJCorr.stan')

Data = {'N': len(X), 'X':X, 'Y':Y, 'Z':Z}

print('MCMC sampling for the correlations started.')


import platform
if platform.system() == 'Windows':
    fit0 = sm0.sampling(data = Data, 
                        pars = ['muX', 'muY', 'muZ', 'sgmX', 'sgmY', 'sgmZ', 'RhoXY', 'RhoXZ', 'RhoYZ'],
                        iter = 5000, n_jobs = 1)
else:
    fit0 = sm0.sampling(data = Data, 
                        pars = ['muX', 'muY', 'muZ', 'sgmX', 'sgmY', 'sgmZ', 'RhoXY', 'RhoXZ', 'RhoYZ'],
                        iter = 5000)

print(fit0)
fout.write('\nMCMC sampling summary (Three variate normal distribution)\n')
fout.write('{}\n'.format(fit0))


print('Compiling the stan script for the partial correlation coefficient.')
sm1 = pystan.StanModel(file = 'PartialCorr.stan')

print('MCMC sampling for the partial correlation coefficient started.')
if platform.system() == 'Windows':
    fit1 = sm1.sampling(data = Data, pars = ['mu1', 'mu2', 'sgm1', 'sgm2', 'sgm_x', 'sgm_y',
                                      'rho', 'Ax', 'Bx', 'Ay', 'By'], iter = 5000, n_jobs = 1)     
else:
    fit1 = sm1.sampling(data = Data, pars = ['mu1', 'mu2', 'sgm1', 'sgm2', 'sgm_x', 'sgm_y',
                                      'rho', 'Ax', 'Bx', 'Ay', 'By'], iter = 5000)     


print(fit1)
fout.write('\nMCMC sampling summary (Partial correlation coefficient)\n')
fout.write('{}\n'.format(fit1))


RhoXY = fit0['RhoXY']
RhoXZ = fit0['RhoXZ']
RhoYZ = fit0['RhoYZ']
p_corr = fit1['rho']


def CalcMAPEst(samples, a = 0.05, n_points = 10000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)  
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)  
    MAP_Est = coord[map_idx]        
    return MAP_Est, est_pdf[map_idx]

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

plt.subplots(figsize = (9, 6))

xcoord = np.linspace(-1, 1, 1000)
ycoord = ss.gaussian_kde(RhoXY).pdf(xcoord)
r_xy_map = CalcMAPEst(RhoXY)[0]
r_xy_mean = RhoXY.mean()
r_xy_025, r_xy_med, r_xy_975 = np.percentile(RhoXY, [2.5, 50, 97.5])
print('\n相関係数 Rxy')
print('MAP推定値 = {0:.3f}'.format(r_xy_map))
print('   平均値 = {0:.3f}'.format(r_xy_mean))
print('   中央値 = {0:.3f}'.format(r_xy_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_xy_025, r_xy_975))

fout.write('\nCorrelation Coefficient Rxy\n')
fout.write('  MAP Est. = {0:.3f}\n'.format(r_xy_map))
fout.write('  Mean     = {0:.3f}\n'.format(r_xy_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_xy_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_xy_025, r_xy_975))

plt.subplot(2,2,1)
plt.title(('相関係数 ' + r'$\rho_{xy}$') + (': mode={0:.3f}'.format(r_xy_map)),
          fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(p_corr).pdf(xcoord)
p_corr_map = CalcMAPEst(p_corr)[0]
p_corr_mean = p_corr.mean()
p_corr_025, p_corr_med, p_corr_975 = np.percentile(p_corr, [2.5, 50, 97.5])
print('\n偏相関係数 Rxy.z')
print('MAP推定値 = {0:.3f}'.format(p_corr_map))
print('   平均値 = {0:.3f}'.format(p_corr_mean))
print('   中央値 = {0:.3f}'.format(p_corr_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(p_corr_025, p_corr_975))

fout.write('\nPartial Correlation Coefficient Rxy.z\n')
fout.write('  MAP Est. = {0:.3f}\n'.format(p_corr_map))
fout.write('  Mean     = {0:.3f}\n'.format(p_corr_mean))
fout.write('  Median   = {0:.3f}\n'.format(p_corr_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(p_corr_025, p_corr_975))

plt.subplot(2,2,2)
plt.title('偏相関係数 ' + r'$\rho_{XY{\bullet}Z}$' +
          ': mode={0:.3f}'.format(p_corr_map), fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(RhoXZ).pdf(xcoord)
r_xz_map = CalcMAPEst(RhoXZ)[0]
r_xz_mean = RhoXZ.mean()
r_xz_025, r_xz_med, r_xz_975 = np.percentile(RhoXZ, [2.5, 50, 97.5])
print('\n相関係数 Rxz')
print('MAP推定値 = {0:.3f}'.format(r_xz_map))
print('   平均値 = {0:.3f}'.format(r_xz_mean))
print('   中央値 = {0:.3f}'.format(r_xz_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_xz_025, r_xz_975))

fout.write('\nCorrelation Coefficient Rxz\n')
fout.write('  MAP Est. = {0:.3f}\n'.format(r_xz_map))
fout.write('  Mean     = {0:.3f}\n'.format(r_xz_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_xz_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_xz_025, r_xz_975))

plt.subplot(2,2,3)
plt.title(r'$\rho_{xz}$' + ': mode={0:.3f}'.format(r_xz_map),
          fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(RhoYZ).pdf(xcoord)
r_yz_map = CalcMAPEst(RhoYZ)[0]
r_yz_mean = RhoYZ.mean()
r_yz_025, r_yz_med, r_yz_975 = np.percentile(RhoYZ, [2.5, 50, 97.5])
print('\n相関係数 Ryz')
print('MAP推定値 = {0:.3f}'.format(r_yz_map))
print('   平均値 = {0:.3f}'.format(r_yz_mean))
print('   中央値 = {0:.3f}'.format(r_yz_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_yz_025, r_yz_975))

fout.write('\nCorrelation Coefficient Ryz\n')
fout.write('  MAP Est. = {0:.3f}\n'.format(r_yz_map))
fout.write('  Mean     = {0:.3f}\n'.format(r_yz_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_yz_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_yz_025, r_yz_975))

plt.subplot(2,2,4)
plt.title(r'$\rho_{yz}$' + ': mode={0:.3f}'.format(r_yz_map),
          fontsize = 16)
plt.plot(xcoord, ycoord)

plt.tight_layout()

plt.savefig(savefig_nm)
print('\nFile {} of the figure was saved.'.format(savefig_nm))
plt.show()

fout.close()
print('\nFile {} was saved.'.format(fout_nm))
print()
